import type { Source, Target, Node, Bound, Entry } from "@canvas-js/okra";
import { TypedEventEmitter } from "@libp2p/interface";
import { Logger } from "@libp2p/logger";
import type { Signature, Signer, Message, Awaitable } from "@canvas-js/interfaces";
export interface ReadOnlyTransaction {
    getHeads: () => Awaitable<Uint8Array[]>;
    getAncestors: (key: Uint8Array, atOrBefore: number, results: Set<string>) => Awaitable<void>;
    isAncestor: (key: Uint8Array, ancestorKey: Uint8Array, visited?: Set<string>) => Awaitable<boolean>;
    messages: Target;
}
export interface ReadWriteTransaction extends ReadOnlyTransaction {
    insert: (signature: Signature, message: Message, entry?: Entry) => Awaitable<void>;
}
export type GossipLogConsumer<Payload = unknown> = (id: string, signature: Signature, message: Message<Payload>) => Awaitable<void>;
export interface GossipLogInit<Payload = unknown> {
    topic: string;
    apply: GossipLogConsumer<Payload>;
    validatePayload?: (payload: unknown) => payload is Payload;
    verifySignature?: (signature: Signature, message: Message<Payload>) => Awaitable<void>;
    signer?: Signer<Payload>;
    indexAncestors?: boolean;
}
export type GossipLogEvents<Payload = unknown> = {
    message: CustomEvent<{
        id: string;
        signature: Signature;
        message: Message<Payload>;
    }>;
    commit: CustomEvent<{
        root: Node;
    }>;
    sync: CustomEvent<{
        peerId?: string;
        duration: number;
        messageCount: number;
    }>;
    error: CustomEvent<{
        error: Error;
    }>;
};
export declare abstract class AbstractGossipLog<Payload = unknown> extends TypedEventEmitter<GossipLogEvents<Payload>> {
    #private;
    readonly topic: string;
    readonly indexAncestors: boolean;
    readonly signer: Signer<Payload>;
    abstract close(): Promise<void>;
    abstract read<T>(callback: (txn: ReadOnlyTransaction) => Awaitable<T>, options?: {
        targetId?: string;
    }): Promise<T>;
    abstract write<T>(callback: (txn: ReadWriteTransaction) => Awaitable<T>, options?: {
        sourceId?: string;
    }): Promise<T>;
    protected abstract entries(lowerBound?: Bound<Uint8Array> | null, upperBound?: Bound<Uint8Array> | null, options?: {
        reverse?: boolean;
    }): AsyncIterable<[key: Uint8Array, value: Uint8Array]>;
    protected readonly log: Logger;
    readonly validatePayload: (payload: unknown) => payload is Payload;
    readonly verifySignature: (signature: Signature, message: Message<Payload>) => Awaitable<void>;
    protected constructor(init: GossipLogInit<Payload>);
    replay(): Promise<void>;
    iterate(lowerBound?: {
        id: string;
        inclusive: boolean;
    } | null, upperBound?: {
        id: string;
        inclusive: boolean;
    } | null, options?: {
        reverse?: boolean;
    }): AsyncIterable<[id: string, signature: Signature, message: Message<Payload>]>;
    encode(signature: Signature, message: Message<Payload>): [key: Uint8Array, value: Uint8Array];
    decode(value: Uint8Array): [id: string, signature: Signature, message: Message<Payload>];
    getClock(): Promise<[clock: number, heads: string[]]>;
    has(id: string): Promise<boolean>;
    get(id: string): Promise<[signature: Signature, message: Message<Payload>] | [null, null]>;
    /**
     * Sign and append a new *unsigned* message to the end of the log.
     * The currently unmerged heads of the local log are used as parents.
     */
    append(txn: ReadWriteTransaction, payload: Payload, options?: {
        signer?: Signer<Payload>;
    }): Promise<{
        id: string;
        signature: Signature;
        message: Message<Payload>;
    }>;
    getMissingParents(txn: ReadWriteTransaction, parents: string[]): Promise<Set<string>>;
    /**
     * Insert an existing signed message into the log (ie received via HTTP API).
     * If any of the parents are not present, throw an error.
     */
    insert(txn: ReadWriteTransaction, signature: Signature, message: Message<Payload>, [key, value]?: [key: Uint8Array, value: Uint8Array]): Promise<{
        id: string;
    }>;
    apply(txn: ReadWriteTransaction, id: string, signature: Signature, message: Message<Payload>, [key, value]?: Entry): Promise<void>;
    getAncestors(txn: ReadOnlyTransaction, id: string, atOrBefore: number): Promise<string[]>;
    isAncestor(txn: ReadOnlyTransaction, id: string, ancestor: string): Promise<boolean>;
    /**
     * Sync with a remote source, applying and inserting all missing messages into the local log
     */
    sync(txn: ReadWriteTransaction, source: Source, options?: {
        sourceId?: string;
    }): AsyncIterable<[string, Signature, Message<Payload>, Entry]>;
    /**
     * Serve a read-only snapshot of the merkle tree
     */
    serve(callback: (source: Source) => Awaitable<void>, options?: {
        targetId?: string;
    }): Promise<void>;
}
